""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: dnsfilter/profile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ProfileFtgddnsFiltersItem(TypedDict, total=False):
    """Nested item for ftgd-dns.filters field."""
    id: int
    category: int
    action: Literal["block", "monitor"]
    log: Literal["enable", "disable"]


class ProfileDomainfilterDict(TypedDict, total=False):
    """Nested object type for domain-filter field."""
    domain_filter_table: int


class ProfileFtgddnsDict(TypedDict, total=False):
    """Nested object type for ftgd-dns field."""
    options: Literal["error-allow", "ftgd-disable"]
    filters: str | list[str] | list[ProfileFtgddnsFiltersItem]


class ProfileExternalipblocklistItem(TypedDict, total=False):
    """Nested item for external-ip-blocklist field."""
    name: str


class ProfileDnstranslationItem(TypedDict, total=False):
    """Nested item for dns-translation field."""
    id: int
    addr_type: Literal["ipv4", "ipv6"]
    src: str
    dst: str
    netmask: str
    status: Literal["enable", "disable"]
    src6: str
    dst6: str
    prefix: int


class ProfileTransparentdnsdatabaseItem(TypedDict, total=False):
    """Nested item for transparent-dns-database field."""
    name: str


class ProfilePayload(TypedDict, total=False):
    """Payload type for Profile operations."""
    name: str
    comment: str
    domain_filter: ProfileDomainfilterDict
    ftgd_dns: ProfileFtgddnsDict
    log_all_domain: Literal["enable", "disable"]
    sdns_ftgd_err_log: Literal["enable", "disable"]
    sdns_domain_log: Literal["enable", "disable"]
    block_action: Literal["block", "redirect", "block-sevrfail"]
    redirect_portal: str
    redirect_portal6: str
    block_botnet: Literal["disable", "enable"]
    safe_search: Literal["disable", "enable"]
    youtube_restrict: Literal["strict", "moderate", "none"]
    external_ip_blocklist: str | list[str] | list[ProfileExternalipblocklistItem]
    dns_translation: str | list[str] | list[ProfileDnstranslationItem]
    transparent_dns_database: str | list[str] | list[ProfileTransparentdnsdatabaseItem]
    strip_ech: Literal["disable", "enable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ProfileResponse(TypedDict, total=False):
    """Response type for Profile - use with .dict property for typed dict access."""
    name: str
    comment: str
    domain_filter: ProfileDomainfilterDict
    ftgd_dns: ProfileFtgddnsDict
    log_all_domain: Literal["enable", "disable"]
    sdns_ftgd_err_log: Literal["enable", "disable"]
    sdns_domain_log: Literal["enable", "disable"]
    block_action: Literal["block", "redirect", "block-sevrfail"]
    redirect_portal: str
    redirect_portal6: str
    block_botnet: Literal["disable", "enable"]
    safe_search: Literal["disable", "enable"]
    youtube_restrict: Literal["strict", "moderate", "none"]
    external_ip_blocklist: list[ProfileExternalipblocklistItem]
    dns_translation: list[ProfileDnstranslationItem]
    transparent_dns_database: list[ProfileTransparentdnsdatabaseItem]
    strip_ech: Literal["disable", "enable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ProfileExternalipblocklistItemObject(FortiObject[ProfileExternalipblocklistItem]):
    """Typed object for external-ip-blocklist table items with attribute access."""
    name: str


class ProfileDnstranslationItemObject(FortiObject[ProfileDnstranslationItem]):
    """Typed object for dns-translation table items with attribute access."""
    id: int
    addr_type: Literal["ipv4", "ipv6"]
    src: str
    dst: str
    netmask: str
    status: Literal["enable", "disable"]
    src6: str
    dst6: str
    prefix: int


class ProfileTransparentdnsdatabaseItemObject(FortiObject[ProfileTransparentdnsdatabaseItem]):
    """Typed object for transparent-dns-database table items with attribute access."""
    name: str


class ProfileFtgddnsFiltersItemObject(FortiObject[ProfileFtgddnsFiltersItem]):
    """Typed object for ftgd-dns.filters table items with attribute access."""
    id: int
    category: int
    action: Literal["block", "monitor"]
    log: Literal["enable", "disable"]


class ProfileDomainfilterObject(FortiObject):
    """Nested object for domain-filter field with attribute access."""
    domain_filter_table: int


class ProfileFtgddnsObject(FortiObject):
    """Nested object for ftgd-dns field with attribute access."""
    options: Literal["error-allow", "ftgd-disable"]
    filters: str | list[str]


class ProfileObject(FortiObject):
    """Typed FortiObject for Profile with field access."""
    name: str
    comment: str
    domain_filter: ProfileDomainfilterObject
    ftgd_dns: ProfileFtgddnsObject
    log_all_domain: Literal["enable", "disable"]
    sdns_ftgd_err_log: Literal["enable", "disable"]
    sdns_domain_log: Literal["enable", "disable"]
    block_action: Literal["block", "redirect", "block-sevrfail"]
    redirect_portal: str
    redirect_portal6: str
    block_botnet: Literal["disable", "enable"]
    safe_search: Literal["disable", "enable"]
    youtube_restrict: Literal["strict", "moderate", "none"]
    external_ip_blocklist: FortiObjectList[ProfileExternalipblocklistItemObject]
    dns_translation: FortiObjectList[ProfileDnstranslationItemObject]
    transparent_dns_database: FortiObjectList[ProfileTransparentdnsdatabaseItemObject]
    strip_ech: Literal["disable", "enable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Profile:
    """
    
    Endpoint: dnsfilter/profile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ProfileObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        domain_filter: ProfileDomainfilterDict | None = ...,
        ftgd_dns: ProfileFtgddnsDict | None = ...,
        log_all_domain: Literal["enable", "disable"] | None = ...,
        sdns_ftgd_err_log: Literal["enable", "disable"] | None = ...,
        sdns_domain_log: Literal["enable", "disable"] | None = ...,
        block_action: Literal["block", "redirect", "block-sevrfail"] | None = ...,
        redirect_portal: str | None = ...,
        redirect_portal6: str | None = ...,
        block_botnet: Literal["disable", "enable"] | None = ...,
        safe_search: Literal["disable", "enable"] | None = ...,
        youtube_restrict: Literal["strict", "moderate", "none"] | None = ...,
        external_ip_blocklist: str | list[str] | list[ProfileExternalipblocklistItem] | None = ...,
        dns_translation: str | list[str] | list[ProfileDnstranslationItem] | None = ...,
        transparent_dns_database: str | list[str] | list[ProfileTransparentdnsdatabaseItem] | None = ...,
        strip_ech: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        domain_filter: ProfileDomainfilterDict | None = ...,
        ftgd_dns: ProfileFtgddnsDict | None = ...,
        log_all_domain: Literal["enable", "disable"] | None = ...,
        sdns_ftgd_err_log: Literal["enable", "disable"] | None = ...,
        sdns_domain_log: Literal["enable", "disable"] | None = ...,
        block_action: Literal["block", "redirect", "block-sevrfail"] | None = ...,
        redirect_portal: str | None = ...,
        redirect_portal6: str | None = ...,
        block_botnet: Literal["disable", "enable"] | None = ...,
        safe_search: Literal["disable", "enable"] | None = ...,
        youtube_restrict: Literal["strict", "moderate", "none"] | None = ...,
        external_ip_blocklist: str | list[str] | list[ProfileExternalipblocklistItem] | None = ...,
        dns_translation: str | list[str] | list[ProfileDnstranslationItem] | None = ...,
        transparent_dns_database: str | list[str] | list[ProfileTransparentdnsdatabaseItem] | None = ...,
        strip_ech: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        domain_filter: ProfileDomainfilterDict | None = ...,
        ftgd_dns: ProfileFtgddnsDict | None = ...,
        log_all_domain: Literal["enable", "disable"] | None = ...,
        sdns_ftgd_err_log: Literal["enable", "disable"] | None = ...,
        sdns_domain_log: Literal["enable", "disable"] | None = ...,
        block_action: Literal["block", "redirect", "block-sevrfail"] | None = ...,
        redirect_portal: str | None = ...,
        redirect_portal6: str | None = ...,
        block_botnet: Literal["disable", "enable"] | None = ...,
        safe_search: Literal["disable", "enable"] | None = ...,
        youtube_restrict: Literal["strict", "moderate", "none"] | None = ...,
        external_ip_blocklist: str | list[str] | list[ProfileExternalipblocklistItem] | None = ...,
        dns_translation: str | list[str] | list[ProfileDnstranslationItem] | None = ...,
        transparent_dns_database: str | list[str] | list[ProfileTransparentdnsdatabaseItem] | None = ...,
        strip_ech: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Profile",
    "ProfilePayload",
    "ProfileResponse",
    "ProfileObject",
]


"""
Pydantic Models for CMDB - wireless_controller/arrp_profile

Runtime validation models for wireless_controller/arrp_profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ArrpProfileDarrpOptimizeSchedules(BaseModel):
    """
    Child table model for darrp-optimize-schedules.
    
    Firewall schedules for DARRP running time. DARRP will run periodically based on darrp-optimize within the schedules. Separate multiple schedule names with a space.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=35, description="Schedule name.")  # datasource: ['firewall.schedule.group.name', 'firewall.schedule.recurring.name', 'firewall.schedule.onetime.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class ArrpProfileModel(BaseModel):
    """
    Pydantic model for wireless_controller/arrp_profile configuration.
    
    Configure WiFi Automatic Radio Resource Provisioning (ARRP) profiles.
    
    Validation Rules:        - name: max_length=35 pattern=        - comment: max_length=255 pattern=        - selection_period: min=0 max=65535 pattern=        - monitor_period: min=0 max=65535 pattern=        - weight_managed_ap: min=0 max=2000 pattern=        - weight_rogue_ap: min=0 max=2000 pattern=        - weight_noise_floor: min=0 max=2000 pattern=        - weight_channel_load: min=0 max=2000 pattern=        - weight_spectral_rssi: min=0 max=2000 pattern=        - weight_weather_channel: min=0 max=2000 pattern=        - weight_dfs_channel: min=0 max=2000 pattern=        - threshold_ap: min=0 max=500 pattern=        - threshold_noise_floor: max_length=7 pattern=        - threshold_channel_load: min=0 max=100 pattern=        - threshold_spectral_rssi: max_length=7 pattern=        - threshold_tx_retries: min=0 max=1000 pattern=        - threshold_rx_errors: min=0 max=100 pattern=        - include_weather_channel: pattern=        - include_dfs_channel: pattern=        - override_darrp_optimize: pattern=        - darrp_optimize: min=0 max=86400 pattern=        - darrp_optimize_schedules: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="WiFi ARRP profile name.")    
    comment: str | None = Field(max_length=255, default=None, description="Comment.")    
    selection_period: int | None = Field(ge=0, le=65535, default=3600, description="Period in seconds to measure average channel load, noise floor, spectral RSSI (default = 3600).")    
    monitor_period: int | None = Field(ge=0, le=65535, default=300, description="Period in seconds to measure average transmit retries and receive errors (default = 300).")    
    weight_managed_ap: int | None = Field(ge=0, le=2000, default=50, description="Weight in DARRP channel score calculation for managed APs (0 - 2000, default = 50).")    
    weight_rogue_ap: int | None = Field(ge=0, le=2000, default=10, description="Weight in DARRP channel score calculation for rogue APs (0 - 2000, default = 10).")    
    weight_noise_floor: int | None = Field(ge=0, le=2000, default=40, description="Weight in DARRP channel score calculation for noise floor (0 - 2000, default = 40).")    
    weight_channel_load: int | None = Field(ge=0, le=2000, default=20, description="Weight in DARRP channel score calculation for channel load (0 - 2000, default = 20).")    
    weight_spectral_rssi: int | None = Field(ge=0, le=2000, default=40, description="Weight in DARRP channel score calculation for spectral RSSI (0 - 2000, default = 40).")    
    weight_weather_channel: int | None = Field(ge=0, le=2000, default=0, description="Weight in DARRP channel score calculation for weather channel (0 - 2000, default = 0).")    
    weight_dfs_channel: int | None = Field(ge=0, le=2000, default=0, description="Weight in DARRP channel score calculation for DFS channel (0 - 2000, default = 0).")    
    threshold_ap: int | None = Field(ge=0, le=500, default=250, description="Threshold to reject channel in DARRP channel selection phase 1 due to surrounding APs (0 - 500, default = 250).")    
    threshold_noise_floor: str | None = Field(max_length=7, default="-85", description="Threshold in dBm to reject channel in DARRP channel selection phase 1 due to noise floor (-95 to -20, default = -85).")    
    threshold_channel_load: int | None = Field(ge=0, le=100, default=60, description="Threshold in percentage to reject channel in DARRP channel selection phase 1 due to channel load (0 - 100, default = 60).")    
    threshold_spectral_rssi: str | None = Field(max_length=7, default="-65", description="Threshold in dBm to reject channel in DARRP channel selection phase 1 due to spectral RSSI (-95 to -20, default = -65).")    
    threshold_tx_retries: int | None = Field(ge=0, le=1000, default=300, description="Threshold in percentage for transmit retries to trigger channel reselection in DARRP monitor stage (0 - 1000, default = 300).")    
    threshold_rx_errors: int | None = Field(ge=0, le=100, default=50, description="Threshold in percentage for receive errors to trigger channel reselection in DARRP monitor stage (0 - 100, default = 50).")    
    include_weather_channel: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable use of weather channel in DARRP channel selection phase 1 (default = enable).")    
    include_dfs_channel: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable use of DFS channel in DARRP channel selection phase 1 (default = enable).")    
    override_darrp_optimize: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to override setting darrp-optimize and darrp-optimize-schedules (default = disable).")    
    darrp_optimize: int | None = Field(ge=0, le=86400, default=86400, description="Time for running Distributed Automatic Radio Resource Provisioning (DARRP) optimizations (0 - 86400 sec, default = 86400, 0 = disable).")    
    darrp_optimize_schedules: list[ArrpProfileDarrpOptimizeSchedules] = Field(default_factory=list, description="Firewall schedules for DARRP running time. DARRP will run periodically based on darrp-optimize within the schedules. Separate multiple schedule names with a space.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ArrpProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_darrp_optimize_schedules_references(self, client: Any) -> list[str]:
        """
        Validate darrp_optimize_schedules references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/schedule/group        - firewall/schedule/recurring        - firewall/schedule/onetime        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ArrpProfileModel(
            ...     darrp_optimize_schedules=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_darrp_optimize_schedules_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.arrp_profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "darrp_optimize_schedules", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.schedule.group.exists(value):
                found = True
            elif await client.api.cmdb.firewall.schedule.recurring.exists(value):
                found = True
            elif await client.api.cmdb.firewall.schedule.onetime.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Darrp-Optimize-Schedules '{value}' not found in "
                    "firewall/schedule/group or firewall/schedule/recurring or firewall/schedule/onetime"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_darrp_optimize_schedules_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ArrpProfileModel",    "ArrpProfileDarrpOptimizeSchedules",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.977310Z
# ============================================================================


"""
Pydantic Models for MONITOR - system/firmware/upgrade

Runtime validation models for system/firmware/upgrade configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class UpgradeSourceEnum(str, Enum):
    """Allowed values for source field."""
    UPLOAD = "upload"
    USB = "usb"
    FORTIGUARD = "fortiguard"
    URL = "url"


# ============================================================================
# Main Model
# ============================================================================

class UpgradeModel(BaseModel):
    """
    Pydantic model for system/firmware/upgrade configuration.
    
    Upgrade firmware image on this device.
    
    Validation Rules:        - source: pattern=        - url: pattern=        - passphrase: pattern=        - force: pattern=        - filename: pattern=        - format_partition: pattern=        - ignore_invalid_signature: pattern=        - file_id: pattern=        - ignore_admin_lockout_upon_downgrade: pattern=        - file_content: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    source: UpgradeSourceEnum = Field()    
    url: str | None = Field(default=None)    
    passphrase: str | None = Field(default=None)    
    force: Any = Field(default=None)    
    filename: str | None = Field(default=None)    
    format_partition: Any = Field(default=None)    
    ignore_invalid_signature: Any = Field(default=None)    
    file_id: str | None = Field(default=None)    
    ignore_admin_lockout_upon_downgrade: Any = Field(default=None)    
    file_content: str | None = Field(default=None)    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "UpgradeModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "UpgradeModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-21T16:24:03.695706Z
# ============================================================================


"""Type stubs for NETWORK category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .connect import Connect
    from .list import List
    from .scan import Scan
    from .status import Status

__all__ = [
    "Connect",
    "List",
    "Scan",
    "Status",
    "Network",
]


class Network:
    """NETWORK API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    connect: Connect
    list: List
    scan: Scan
    status: Status

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize network category with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: vpn/ipsec/phase2
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class Phase2Payload(TypedDict, total=False):
    """Payload type for Phase2 operations."""
    name: str
    phase1name: str
    dhcp_ipsec: Literal["enable", "disable"]
    use_natip: Literal["enable", "disable"]
    selector_match: Literal["exact", "subset", "auto"]
    proposal: str | list[str]
    pfs: Literal["enable", "disable"]
    dhgrp: str | list[str]
    addke1: str | list[str]
    addke2: str | list[str]
    addke3: str | list[str]
    addke4: str | list[str]
    addke5: str | list[str]
    addke6: str | list[str]
    addke7: str | list[str]
    replay: Literal["enable", "disable"]
    keepalive: Literal["enable", "disable"]
    auto_negotiate: Literal["enable", "disable"]
    add_route: Literal["phase1", "enable", "disable"]
    inbound_dscp_copy: Literal["phase1", "enable", "disable"]
    keylifeseconds: int
    keylifekbs: int
    keylife_type: Literal["seconds", "kbs", "both"]
    single_source: Literal["enable", "disable"]
    route_overlap: Literal["use-old", "use-new", "allow"]
    encapsulation: Literal["tunnel-mode", "transport-mode"]
    l2tp: Literal["enable", "disable"]
    comments: str
    initiator_ts_narrow: Literal["enable", "disable"]
    diffserv: Literal["enable", "disable"]
    diffservcode: str
    protocol: int
    src_name: str
    src_name6: str
    src_addr_type: Literal["subnet", "range", "ip", "name"]
    src_start_ip: str
    src_start_ip6: str
    src_end_ip: str
    src_end_ip6: str
    src_subnet: str
    src_subnet6: str
    src_port: int
    dst_name: str
    dst_name6: str
    dst_addr_type: Literal["subnet", "range", "ip", "name"]
    dst_start_ip: str
    dst_start_ip6: str
    dst_end_ip: str
    dst_end_ip6: str
    dst_subnet: str
    dst_subnet6: str
    dst_port: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class Phase2Response(TypedDict, total=False):
    """Response type for Phase2 - use with .dict property for typed dict access."""
    name: str
    phase1name: str
    dhcp_ipsec: Literal["enable", "disable"]
    use_natip: Literal["enable", "disable"]
    selector_match: Literal["exact", "subset", "auto"]
    proposal: str
    pfs: Literal["enable", "disable"]
    dhgrp: str
    addke1: str
    addke2: str
    addke3: str
    addke4: str
    addke5: str
    addke6: str
    addke7: str
    replay: Literal["enable", "disable"]
    keepalive: Literal["enable", "disable"]
    auto_negotiate: Literal["enable", "disable"]
    add_route: Literal["phase1", "enable", "disable"]
    inbound_dscp_copy: Literal["phase1", "enable", "disable"]
    keylifeseconds: int
    keylifekbs: int
    keylife_type: Literal["seconds", "kbs", "both"]
    single_source: Literal["enable", "disable"]
    route_overlap: Literal["use-old", "use-new", "allow"]
    encapsulation: Literal["tunnel-mode", "transport-mode"]
    l2tp: Literal["enable", "disable"]
    comments: str
    initiator_ts_narrow: Literal["enable", "disable"]
    diffserv: Literal["enable", "disable"]
    diffservcode: str
    protocol: int
    src_name: str
    src_name6: str
    src_addr_type: Literal["subnet", "range", "ip", "name"]
    src_start_ip: str
    src_start_ip6: str
    src_end_ip: str
    src_end_ip6: str
    src_subnet: str
    src_subnet6: str
    src_port: int
    dst_name: str
    dst_name6: str
    dst_addr_type: Literal["subnet", "range", "ip", "name"]
    dst_start_ip: str
    dst_start_ip6: str
    dst_end_ip: str
    dst_end_ip6: str
    dst_subnet: str
    dst_subnet6: str
    dst_port: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class Phase2Object(FortiObject):
    """Typed FortiObject for Phase2 with field access."""
    name: str
    phase1name: str
    dhcp_ipsec: Literal["enable", "disable"]
    use_natip: Literal["enable", "disable"]
    selector_match: Literal["exact", "subset", "auto"]
    proposal: str
    pfs: Literal["enable", "disable"]
    dhgrp: str
    addke1: str
    addke2: str
    addke3: str
    addke4: str
    addke5: str
    addke6: str
    addke7: str
    replay: Literal["enable", "disable"]
    keepalive: Literal["enable", "disable"]
    auto_negotiate: Literal["enable", "disable"]
    add_route: Literal["phase1", "enable", "disable"]
    inbound_dscp_copy: Literal["phase1", "enable", "disable"]
    keylifeseconds: int
    keylifekbs: int
    keylife_type: Literal["seconds", "kbs", "both"]
    single_source: Literal["enable", "disable"]
    route_overlap: Literal["use-old", "use-new", "allow"]
    encapsulation: Literal["tunnel-mode", "transport-mode"]
    l2tp: Literal["enable", "disable"]
    comments: str
    initiator_ts_narrow: Literal["enable", "disable"]
    diffserv: Literal["enable", "disable"]
    diffservcode: str
    protocol: int
    src_name: str
    src_name6: str
    src_addr_type: Literal["subnet", "range", "ip", "name"]
    src_start_ip: str
    src_start_ip6: str
    src_end_ip: str
    src_end_ip6: str
    src_subnet: str
    src_subnet6: str
    src_port: int
    dst_name: str
    dst_name6: str
    dst_addr_type: Literal["subnet", "range", "ip", "name"]
    dst_start_ip: str
    dst_start_ip6: str
    dst_end_ip: str
    dst_end_ip6: str
    dst_subnet: str
    dst_subnet6: str
    dst_port: int


# ================================================================
# Main Endpoint Class
# ================================================================

class Phase2:
    """
    
    Endpoint: vpn/ipsec/phase2
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Phase2Object: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[Phase2Object]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: Phase2Payload | None = ...,
        name: str | None = ...,
        phase1name: str | None = ...,
        dhcp_ipsec: Literal["enable", "disable"] | None = ...,
        use_natip: Literal["enable", "disable"] | None = ...,
        selector_match: Literal["exact", "subset", "auto"] | None = ...,
        proposal: str | list[str] | None = ...,
        pfs: Literal["enable", "disable"] | None = ...,
        dhgrp: str | list[str] | None = ...,
        addke1: str | list[str] | None = ...,
        addke2: str | list[str] | None = ...,
        addke3: str | list[str] | None = ...,
        addke4: str | list[str] | None = ...,
        addke5: str | list[str] | None = ...,
        addke6: str | list[str] | None = ...,
        addke7: str | list[str] | None = ...,
        replay: Literal["enable", "disable"] | None = ...,
        keepalive: Literal["enable", "disable"] | None = ...,
        auto_negotiate: Literal["enable", "disable"] | None = ...,
        add_route: Literal["phase1", "enable", "disable"] | None = ...,
        inbound_dscp_copy: Literal["phase1", "enable", "disable"] | None = ...,
        keylifeseconds: int | None = ...,
        keylifekbs: int | None = ...,
        keylife_type: Literal["seconds", "kbs", "both"] | None = ...,
        single_source: Literal["enable", "disable"] | None = ...,
        route_overlap: Literal["use-old", "use-new", "allow"] | None = ...,
        encapsulation: Literal["tunnel-mode", "transport-mode"] | None = ...,
        l2tp: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        initiator_ts_narrow: Literal["enable", "disable"] | None = ...,
        diffserv: Literal["enable", "disable"] | None = ...,
        diffservcode: str | None = ...,
        protocol: int | None = ...,
        src_name: str | None = ...,
        src_name6: str | None = ...,
        src_addr_type: Literal["subnet", "range", "ip", "name"] | None = ...,
        src_start_ip: str | None = ...,
        src_start_ip6: str | None = ...,
        src_end_ip: str | None = ...,
        src_end_ip6: str | None = ...,
        src_subnet: str | None = ...,
        src_subnet6: str | None = ...,
        src_port: int | None = ...,
        dst_name: str | None = ...,
        dst_name6: str | None = ...,
        dst_addr_type: Literal["subnet", "range", "ip", "name"] | None = ...,
        dst_start_ip: str | None = ...,
        dst_start_ip6: str | None = ...,
        dst_end_ip: str | None = ...,
        dst_end_ip6: str | None = ...,
        dst_subnet: str | None = ...,
        dst_subnet6: str | None = ...,
        dst_port: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Phase2Object: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: Phase2Payload | None = ...,
        name: str | None = ...,
        phase1name: str | None = ...,
        dhcp_ipsec: Literal["enable", "disable"] | None = ...,
        use_natip: Literal["enable", "disable"] | None = ...,
        selector_match: Literal["exact", "subset", "auto"] | None = ...,
        proposal: str | list[str] | None = ...,
        pfs: Literal["enable", "disable"] | None = ...,
        dhgrp: str | list[str] | None = ...,
        addke1: str | list[str] | None = ...,
        addke2: str | list[str] | None = ...,
        addke3: str | list[str] | None = ...,
        addke4: str | list[str] | None = ...,
        addke5: str | list[str] | None = ...,
        addke6: str | list[str] | None = ...,
        addke7: str | list[str] | None = ...,
        replay: Literal["enable", "disable"] | None = ...,
        keepalive: Literal["enable", "disable"] | None = ...,
        auto_negotiate: Literal["enable", "disable"] | None = ...,
        add_route: Literal["phase1", "enable", "disable"] | None = ...,
        inbound_dscp_copy: Literal["phase1", "enable", "disable"] | None = ...,
        keylifeseconds: int | None = ...,
        keylifekbs: int | None = ...,
        keylife_type: Literal["seconds", "kbs", "both"] | None = ...,
        single_source: Literal["enable", "disable"] | None = ...,
        route_overlap: Literal["use-old", "use-new", "allow"] | None = ...,
        encapsulation: Literal["tunnel-mode", "transport-mode"] | None = ...,
        l2tp: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        initiator_ts_narrow: Literal["enable", "disable"] | None = ...,
        diffserv: Literal["enable", "disable"] | None = ...,
        diffservcode: str | None = ...,
        protocol: int | None = ...,
        src_name: str | None = ...,
        src_name6: str | None = ...,
        src_addr_type: Literal["subnet", "range", "ip", "name"] | None = ...,
        src_start_ip: str | None = ...,
        src_start_ip6: str | None = ...,
        src_end_ip: str | None = ...,
        src_end_ip6: str | None = ...,
        src_subnet: str | None = ...,
        src_subnet6: str | None = ...,
        src_port: int | None = ...,
        dst_name: str | None = ...,
        dst_name6: str | None = ...,
        dst_addr_type: Literal["subnet", "range", "ip", "name"] | None = ...,
        dst_start_ip: str | None = ...,
        dst_start_ip6: str | None = ...,
        dst_end_ip: str | None = ...,
        dst_end_ip6: str | None = ...,
        dst_subnet: str | None = ...,
        dst_subnet6: str | None = ...,
        dst_port: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Phase2Object: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: Phase2Payload | None = ...,
        name: str | None = ...,
        phase1name: str | None = ...,
        dhcp_ipsec: Literal["enable", "disable"] | None = ...,
        use_natip: Literal["enable", "disable"] | None = ...,
        selector_match: Literal["exact", "subset", "auto"] | None = ...,
        proposal: Literal["null-md5", "null-sha1", "null-sha256", "null-sha384", "null-sha512", "des-null", "des-md5", "des-sha1", "des-sha256", "des-sha384", "des-sha512", "3des-null", "3des-md5", "3des-sha1", "3des-sha256", "3des-sha384", "3des-sha512", "aes128-null", "aes128-md5", "aes128-sha1", "aes128-sha256", "aes128-sha384", "aes128-sha512", "aes128gcm", "aes192-null", "aes192-md5", "aes192-sha1", "aes192-sha256", "aes192-sha384", "aes192-sha512", "aes256-null", "aes256-md5", "aes256-sha1", "aes256-sha256", "aes256-sha384", "aes256-sha512", "aes256gcm", "chacha20poly1305", "aria128-null", "aria128-md5", "aria128-sha1", "aria128-sha256", "aria128-sha384", "aria128-sha512", "aria192-null", "aria192-md5", "aria192-sha1", "aria192-sha256", "aria192-sha384", "aria192-sha512", "aria256-null", "aria256-md5", "aria256-sha1", "aria256-sha256", "aria256-sha384", "aria256-sha512", "seed-null", "seed-md5", "seed-sha1", "seed-sha256", "seed-sha384", "seed-sha512"] | list[str] | None = ...,
        pfs: Literal["enable", "disable"] | None = ...,
        dhgrp: Literal["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31", "32"] | list[str] | None = ...,
        addke1: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke2: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke3: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke4: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke5: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke6: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke7: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        replay: Literal["enable", "disable"] | None = ...,
        keepalive: Literal["enable", "disable"] | None = ...,
        auto_negotiate: Literal["enable", "disable"] | None = ...,
        add_route: Literal["phase1", "enable", "disable"] | None = ...,
        inbound_dscp_copy: Literal["phase1", "enable", "disable"] | None = ...,
        keylifeseconds: int | None = ...,
        keylifekbs: int | None = ...,
        keylife_type: Literal["seconds", "kbs", "both"] | None = ...,
        single_source: Literal["enable", "disable"] | None = ...,
        route_overlap: Literal["use-old", "use-new", "allow"] | None = ...,
        encapsulation: Literal["tunnel-mode", "transport-mode"] | None = ...,
        l2tp: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        initiator_ts_narrow: Literal["enable", "disable"] | None = ...,
        diffserv: Literal["enable", "disable"] | None = ...,
        diffservcode: str | None = ...,
        protocol: int | None = ...,
        src_name: str | None = ...,
        src_name6: str | None = ...,
        src_addr_type: Literal["subnet", "range", "ip", "name"] | None = ...,
        src_start_ip: str | None = ...,
        src_start_ip6: str | None = ...,
        src_end_ip: str | None = ...,
        src_end_ip6: str | None = ...,
        src_subnet: str | None = ...,
        src_subnet6: str | None = ...,
        src_port: int | None = ...,
        dst_name: str | None = ...,
        dst_name6: str | None = ...,
        dst_addr_type: Literal["subnet", "range", "ip", "name"] | None = ...,
        dst_start_ip: str | None = ...,
        dst_start_ip6: str | None = ...,
        dst_end_ip: str | None = ...,
        dst_end_ip6: str | None = ...,
        dst_subnet: str | None = ...,
        dst_subnet6: str | None = ...,
        dst_port: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Phase2",
    "Phase2Payload",
    "Phase2Response",
    "Phase2Object",
]


"""FortiOS CMDB - Ddns category"""

from .lookup import Lookup
from .servers import Servers

__all__ = [
    "Ddns",
    "Lookup",
    "Servers",
]


class Ddns:
    """Ddns endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ddns endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.lookup = Lookup(client)
        self.servers = Servers(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/fortiguard
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class FortiguardPayload(TypedDict, total=False):
    """Payload type for Fortiguard operations."""
    fortiguard_anycast: Literal["enable", "disable"]
    fortiguard_anycast_source: Literal["fortinet", "aws", "debug"]
    protocol: Literal["udp", "http", "https"]
    port: Literal["8888", "53", "80", "443"]
    load_balance_servers: int
    auto_join_forticloud: Literal["enable", "disable"]
    update_server_location: Literal["automatic", "usa", "eu"]
    sandbox_region: str
    sandbox_inline_scan: Literal["enable", "disable"]
    update_ffdb: Literal["enable", "disable"]
    update_uwdb: Literal["enable", "disable"]
    update_dldb: Literal["enable", "disable"]
    update_extdb: Literal["enable", "disable"]
    update_build_proxy: Literal["enable", "disable"]
    persistent_connection: Literal["enable", "disable"]
    vdom: str
    auto_firmware_upgrade: Literal["enable", "disable"]
    auto_firmware_upgrade_day: str | list[str]
    auto_firmware_upgrade_delay: int
    auto_firmware_upgrade_start_hour: int
    auto_firmware_upgrade_end_hour: int
    FDS_license_expiring_days: int
    subscribe_update_notification: Literal["enable", "disable"]
    antispam_force_off: Literal["enable", "disable"]
    antispam_cache: Literal["enable", "disable"]
    antispam_cache_ttl: int
    antispam_cache_mpermille: int
    antispam_license: int
    antispam_expiration: int
    antispam_timeout: int
    outbreak_prevention_force_off: Literal["enable", "disable"]
    outbreak_prevention_cache: Literal["enable", "disable"]
    outbreak_prevention_cache_ttl: int
    outbreak_prevention_cache_mpermille: int
    outbreak_prevention_license: int
    outbreak_prevention_expiration: int
    outbreak_prevention_timeout: int
    webfilter_force_off: Literal["enable", "disable"]
    webfilter_cache: Literal["enable", "disable"]
    webfilter_cache_ttl: int
    webfilter_license: int
    webfilter_expiration: int
    webfilter_timeout: int
    sdns_server_ip: str | list[str]
    sdns_server_port: int
    anycast_sdns_server_ip: str
    anycast_sdns_server_port: int
    sdns_options: str | list[str]
    source_ip: str
    source_ip6: str
    proxy_server_ip: str
    proxy_server_port: int
    proxy_username: str
    proxy_password: str
    ddns_server_ip: str
    ddns_server_ip6: str
    ddns_server_port: int
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    vrf_select: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class FortiguardResponse(TypedDict, total=False):
    """Response type for Fortiguard - use with .dict property for typed dict access."""
    fortiguard_anycast: Literal["enable", "disable"]
    fortiguard_anycast_source: Literal["fortinet", "aws", "debug"]
    protocol: Literal["udp", "http", "https"]
    port: Literal["8888", "53", "80", "443"]
    load_balance_servers: int
    auto_join_forticloud: Literal["enable", "disable"]
    update_server_location: Literal["automatic", "usa", "eu"]
    sandbox_region: str
    sandbox_inline_scan: Literal["enable", "disable"]
    update_ffdb: Literal["enable", "disable"]
    update_uwdb: Literal["enable", "disable"]
    update_dldb: Literal["enable", "disable"]
    update_extdb: Literal["enable", "disable"]
    update_build_proxy: Literal["enable", "disable"]
    persistent_connection: Literal["enable", "disable"]
    vdom: str
    auto_firmware_upgrade: Literal["enable", "disable"]
    auto_firmware_upgrade_day: str
    auto_firmware_upgrade_delay: int
    auto_firmware_upgrade_start_hour: int
    auto_firmware_upgrade_end_hour: int
    FDS_license_expiring_days: int
    subscribe_update_notification: Literal["enable", "disable"]
    antispam_force_off: Literal["enable", "disable"]
    antispam_cache: Literal["enable", "disable"]
    antispam_cache_ttl: int
    antispam_cache_mpermille: int
    antispam_license: int
    antispam_expiration: int
    antispam_timeout: int
    outbreak_prevention_force_off: Literal["enable", "disable"]
    outbreak_prevention_cache: Literal["enable", "disable"]
    outbreak_prevention_cache_ttl: int
    outbreak_prevention_cache_mpermille: int
    outbreak_prevention_license: int
    outbreak_prevention_expiration: int
    outbreak_prevention_timeout: int
    webfilter_force_off: Literal["enable", "disable"]
    webfilter_cache: Literal["enable", "disable"]
    webfilter_cache_ttl: int
    webfilter_license: int
    webfilter_expiration: int
    webfilter_timeout: int
    sdns_server_ip: str | list[str]
    sdns_server_port: int
    anycast_sdns_server_ip: str
    anycast_sdns_server_port: int
    sdns_options: str
    source_ip: str
    source_ip6: str
    proxy_server_ip: str
    proxy_server_port: int
    proxy_username: str
    proxy_password: str
    ddns_server_ip: str
    ddns_server_ip6: str
    ddns_server_port: int
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    vrf_select: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class FortiguardObject(FortiObject):
    """Typed FortiObject for Fortiguard with field access."""
    fortiguard_anycast: Literal["enable", "disable"]
    fortiguard_anycast_source: Literal["fortinet", "aws", "debug"]
    protocol: Literal["udp", "http", "https"]
    port: Literal["8888", "53", "80", "443"]
    load_balance_servers: int
    auto_join_forticloud: Literal["enable", "disable"]
    update_server_location: Literal["automatic", "usa", "eu"]
    sandbox_region: str
    sandbox_inline_scan: Literal["enable", "disable"]
    update_ffdb: Literal["enable", "disable"]
    update_uwdb: Literal["enable", "disable"]
    update_dldb: Literal["enable", "disable"]
    update_extdb: Literal["enable", "disable"]
    update_build_proxy: Literal["enable", "disable"]
    persistent_connection: Literal["enable", "disable"]
    auto_firmware_upgrade: Literal["enable", "disable"]
    auto_firmware_upgrade_day: str
    auto_firmware_upgrade_delay: int
    auto_firmware_upgrade_start_hour: int
    auto_firmware_upgrade_end_hour: int
    FDS_license_expiring_days: int
    subscribe_update_notification: Literal["enable", "disable"]
    antispam_force_off: Literal["enable", "disable"]
    antispam_cache: Literal["enable", "disable"]
    antispam_cache_ttl: int
    antispam_cache_mpermille: int
    antispam_license: int
    antispam_expiration: int
    antispam_timeout: int
    outbreak_prevention_force_off: Literal["enable", "disable"]
    outbreak_prevention_cache: Literal["enable", "disable"]
    outbreak_prevention_cache_ttl: int
    outbreak_prevention_cache_mpermille: int
    outbreak_prevention_license: int
    outbreak_prevention_expiration: int
    outbreak_prevention_timeout: int
    webfilter_force_off: Literal["enable", "disable"]
    webfilter_cache: Literal["enable", "disable"]
    webfilter_cache_ttl: int
    webfilter_license: int
    webfilter_expiration: int
    webfilter_timeout: int
    sdns_server_ip: str | list[str]
    sdns_server_port: int
    anycast_sdns_server_ip: str
    anycast_sdns_server_port: int
    sdns_options: str
    source_ip: str
    source_ip6: str
    proxy_server_ip: str
    proxy_server_port: int
    proxy_username: str
    proxy_password: str
    ddns_server_ip: str
    ddns_server_ip6: str
    ddns_server_port: int
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    vrf_select: int


# ================================================================
# Main Endpoint Class
# ================================================================

class Fortiguard:
    """
    
    Endpoint: system/fortiguard
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiguardObject: ...
    
    def get_schema(
        self,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: FortiguardPayload | None = ...,
        fortiguard_anycast: Literal["enable", "disable"] | None = ...,
        fortiguard_anycast_source: Literal["fortinet", "aws", "debug"] | None = ...,
        protocol: Literal["udp", "http", "https"] | None = ...,
        port: Literal["8888", "53", "80", "443"] | None = ...,
        load_balance_servers: int | None = ...,
        auto_join_forticloud: Literal["enable", "disable"] | None = ...,
        update_server_location: Literal["automatic", "usa", "eu"] | None = ...,
        sandbox_region: str | None = ...,
        sandbox_inline_scan: Literal["enable", "disable"] | None = ...,
        update_ffdb: Literal["enable", "disable"] | None = ...,
        update_uwdb: Literal["enable", "disable"] | None = ...,
        update_dldb: Literal["enable", "disable"] | None = ...,
        update_extdb: Literal["enable", "disable"] | None = ...,
        update_build_proxy: Literal["enable", "disable"] | None = ...,
        persistent_connection: Literal["enable", "disable"] | None = ...,
        auto_firmware_upgrade: Literal["enable", "disable"] | None = ...,
        auto_firmware_upgrade_day: str | list[str] | None = ...,
        auto_firmware_upgrade_delay: int | None = ...,
        auto_firmware_upgrade_start_hour: int | None = ...,
        auto_firmware_upgrade_end_hour: int | None = ...,
        FDS_license_expiring_days: int | None = ...,
        subscribe_update_notification: Literal["enable", "disable"] | None = ...,
        antispam_force_off: Literal["enable", "disable"] | None = ...,
        antispam_cache: Literal["enable", "disable"] | None = ...,
        antispam_cache_ttl: int | None = ...,
        antispam_cache_mpermille: int | None = ...,
        antispam_license: int | None = ...,
        antispam_expiration: int | None = ...,
        antispam_timeout: int | None = ...,
        outbreak_prevention_force_off: Literal["enable", "disable"] | None = ...,
        outbreak_prevention_cache: Literal["enable", "disable"] | None = ...,
        outbreak_prevention_cache_ttl: int | None = ...,
        outbreak_prevention_cache_mpermille: int | None = ...,
        outbreak_prevention_license: int | None = ...,
        outbreak_prevention_expiration: int | None = ...,
        outbreak_prevention_timeout: int | None = ...,
        webfilter_force_off: Literal["enable", "disable"] | None = ...,
        webfilter_cache: Literal["enable", "disable"] | None = ...,
        webfilter_cache_ttl: int | None = ...,
        webfilter_license: int | None = ...,
        webfilter_expiration: int | None = ...,
        webfilter_timeout: int | None = ...,
        sdns_server_ip: str | list[str] | None = ...,
        sdns_server_port: int | None = ...,
        anycast_sdns_server_ip: str | None = ...,
        anycast_sdns_server_port: int | None = ...,
        sdns_options: str | list[str] | None = ...,
        source_ip: str | None = ...,
        source_ip6: str | None = ...,
        proxy_server_ip: str | None = ...,
        proxy_server_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        ddns_server_ip: str | None = ...,
        ddns_server_ip6: str | None = ...,
        ddns_server_port: int | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiguardObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: FortiguardPayload | None = ...,
        fortiguard_anycast: Literal["enable", "disable"] | None = ...,
        fortiguard_anycast_source: Literal["fortinet", "aws", "debug"] | None = ...,
        protocol: Literal["udp", "http", "https"] | None = ...,
        port: Literal["8888", "53", "80", "443"] | None = ...,
        load_balance_servers: int | None = ...,
        auto_join_forticloud: Literal["enable", "disable"] | None = ...,
        update_server_location: Literal["automatic", "usa", "eu"] | None = ...,
        sandbox_region: str | None = ...,
        sandbox_inline_scan: Literal["enable", "disable"] | None = ...,
        update_ffdb: Literal["enable", "disable"] | None = ...,
        update_uwdb: Literal["enable", "disable"] | None = ...,
        update_dldb: Literal["enable", "disable"] | None = ...,
        update_extdb: Literal["enable", "disable"] | None = ...,
        update_build_proxy: Literal["enable", "disable"] | None = ...,
        persistent_connection: Literal["enable", "disable"] | None = ...,
        auto_firmware_upgrade: Literal["enable", "disable"] | None = ...,
        auto_firmware_upgrade_day: Literal["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"] | list[str] | None = ...,
        auto_firmware_upgrade_delay: int | None = ...,
        auto_firmware_upgrade_start_hour: int | None = ...,
        auto_firmware_upgrade_end_hour: int | None = ...,
        FDS_license_expiring_days: int | None = ...,
        subscribe_update_notification: Literal["enable", "disable"] | None = ...,
        antispam_force_off: Literal["enable", "disable"] | None = ...,
        antispam_cache: Literal["enable", "disable"] | None = ...,
        antispam_cache_ttl: int | None = ...,
        antispam_cache_mpermille: int | None = ...,
        antispam_license: int | None = ...,
        antispam_expiration: int | None = ...,
        antispam_timeout: int | None = ...,
        outbreak_prevention_force_off: Literal["enable", "disable"] | None = ...,
        outbreak_prevention_cache: Literal["enable", "disable"] | None = ...,
        outbreak_prevention_cache_ttl: int | None = ...,
        outbreak_prevention_cache_mpermille: int | None = ...,
        outbreak_prevention_license: int | None = ...,
        outbreak_prevention_expiration: int | None = ...,
        outbreak_prevention_timeout: int | None = ...,
        webfilter_force_off: Literal["enable", "disable"] | None = ...,
        webfilter_cache: Literal["enable", "disable"] | None = ...,
        webfilter_cache_ttl: int | None = ...,
        webfilter_license: int | None = ...,
        webfilter_expiration: int | None = ...,
        webfilter_timeout: int | None = ...,
        sdns_server_ip: str | list[str] | None = ...,
        sdns_server_port: int | None = ...,
        anycast_sdns_server_ip: str | None = ...,
        anycast_sdns_server_port: int | None = ...,
        sdns_options: Literal["include-question-section"] | list[str] | None = ...,
        source_ip: str | None = ...,
        source_ip6: str | None = ...,
        proxy_server_ip: str | None = ...,
        proxy_server_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        ddns_server_ip: str | None = ...,
        ddns_server_ip6: str | None = ...,
        ddns_server_port: int | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Fortiguard",
    "FortiguardPayload",
    "FortiguardResponse",
    "FortiguardObject",
]


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/profile_protocol_options
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ProfileProtocolOptionsCifsServerkeytabItem(TypedDict, total=False):
    """Nested item for cifs.server-keytab field."""
    principal: str
    keytab: str


class ProfileProtocolOptionsHttpDict(TypedDict, total=False):
    """Nested object type for http field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["clientcomfort", "servercomfort", "oversize", "chunkedbypass"]
    comfort_interval: int
    comfort_amount: int
    range_block: Literal["disable", "enable"]
    strip_x_forwarded_for: Literal["disable", "enable"]
    post_lang: Literal["jisx0201", "jisx0208", "jisx0212", "gb2312", "ksc5601-ex", "euc-jp", "sjis", "iso2022-jp", "iso2022-jp-1", "iso2022-jp-2", "euc-cn", "ces-gbk", "hz", "ces-big5", "euc-kr", "iso2022-jp-3", "iso8859-1", "tis620", "cp874", "cp1252", "cp1251"]
    streaming_content_bypass: Literal["enable", "disable"]
    switching_protocols: Literal["bypass", "block"]
    unknown_http_version: Literal["reject", "tunnel", "best-effort"]
    http_0_9: Literal["allow", "block"]
    tunnel_non_http: Literal["enable", "disable"]
    h2c: Literal["enable", "disable"]
    unknown_content_encoding: Literal["block", "inspect", "bypass"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    stream_based_uncompressed_limit: int
    scan_bzip2: Literal["enable", "disable"]
    verify_dns_for_policy_matching: Literal["enable", "disable"]
    block_page_status_code: int
    retry_count: int
    domain_fronting: Literal["allow", "monitor", "block", "strict"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    ssl_offloaded: Literal["no", "yes"]
    address_ip_rating: Literal["enable", "disable"]


class ProfileProtocolOptionsFtpDict(TypedDict, total=False):
    """Nested object type for ftp field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    options: Literal["clientcomfort", "oversize", "splice", "bypass-rest-command", "bypass-mode-command"]
    comfort_interval: int
    comfort_amount: int
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    stream_based_uncompressed_limit: int
    scan_bzip2: Literal["enable", "disable"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    ssl_offloaded: Literal["no", "yes"]
    explicit_ftp_tls: Literal["enable", "disable"]


class ProfileProtocolOptionsImapDict(TypedDict, total=False):
    """Nested object type for imap field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsMapiDict(TypedDict, total=False):
    """Nested object type for mapi field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]


class ProfileProtocolOptionsPop3Dict(TypedDict, total=False):
    """Nested object type for pop3 field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsSmtpDict(TypedDict, total=False):
    """Nested object type for smtp field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize", "splice"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    server_busy: Literal["enable", "disable"]
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsNntpDict(TypedDict, total=False):
    """Nested object type for nntp field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["oversize", "splice"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]


class ProfileProtocolOptionsSshDict(TypedDict, total=False):
    """Nested object type for ssh field."""
    options: Literal["oversize", "clientcomfort", "servercomfort"]
    comfort_interval: int
    comfort_amount: int
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    stream_based_uncompressed_limit: int
    scan_bzip2: Literal["enable", "disable"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsDnsDict(TypedDict, total=False):
    """Nested object type for dns field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]


class ProfileProtocolOptionsCifsDict(TypedDict, total=False):
    """Nested object type for cifs field."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    options: Literal["oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    server_credential_type: Literal["none", "credential-replication", "credential-keytab"]
    domain_controller: str
    server_keytab: str | list[str] | list[ProfileProtocolOptionsCifsServerkeytabItem]


class ProfileProtocolOptionsMailsignatureDict(TypedDict, total=False):
    """Nested object type for mail-signature field."""
    status: Literal["disable", "enable"]
    signature: str


class ProfileProtocolOptionsPayload(TypedDict, total=False):
    """Payload type for ProfileProtocolOptions operations."""
    name: str
    comment: str
    replacemsg_group: str
    oversize_log: Literal["disable", "enable"]
    switching_protocols_log: Literal["disable", "enable"]
    http: ProfileProtocolOptionsHttpDict
    ftp: ProfileProtocolOptionsFtpDict
    imap: ProfileProtocolOptionsImapDict
    mapi: ProfileProtocolOptionsMapiDict
    pop3: ProfileProtocolOptionsPop3Dict
    smtp: ProfileProtocolOptionsSmtpDict
    nntp: ProfileProtocolOptionsNntpDict
    ssh: ProfileProtocolOptionsSshDict
    dns: ProfileProtocolOptionsDnsDict
    cifs: ProfileProtocolOptionsCifsDict
    mail_signature: ProfileProtocolOptionsMailsignatureDict
    rpc_over_http: Literal["enable", "disable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ProfileProtocolOptionsResponse(TypedDict, total=False):
    """Response type for ProfileProtocolOptions - use with .dict property for typed dict access."""
    name: str
    comment: str
    replacemsg_group: str
    oversize_log: Literal["disable", "enable"]
    switching_protocols_log: Literal["disable", "enable"]
    http: ProfileProtocolOptionsHttpDict
    ftp: ProfileProtocolOptionsFtpDict
    imap: ProfileProtocolOptionsImapDict
    mapi: ProfileProtocolOptionsMapiDict
    pop3: ProfileProtocolOptionsPop3Dict
    smtp: ProfileProtocolOptionsSmtpDict
    nntp: ProfileProtocolOptionsNntpDict
    ssh: ProfileProtocolOptionsSshDict
    dns: ProfileProtocolOptionsDnsDict
    cifs: ProfileProtocolOptionsCifsDict
    mail_signature: ProfileProtocolOptionsMailsignatureDict
    rpc_over_http: Literal["enable", "disable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ProfileProtocolOptionsCifsServerkeytabItemObject(FortiObject[ProfileProtocolOptionsCifsServerkeytabItem]):
    """Typed object for cifs.server-keytab table items with attribute access."""
    principal: str
    keytab: str


class ProfileProtocolOptionsHttpObject(FortiObject):
    """Nested object for http field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["clientcomfort", "servercomfort", "oversize", "chunkedbypass"]
    comfort_interval: int
    comfort_amount: int
    range_block: Literal["disable", "enable"]
    strip_x_forwarded_for: Literal["disable", "enable"]
    post_lang: Literal["jisx0201", "jisx0208", "jisx0212", "gb2312", "ksc5601-ex", "euc-jp", "sjis", "iso2022-jp", "iso2022-jp-1", "iso2022-jp-2", "euc-cn", "ces-gbk", "hz", "ces-big5", "euc-kr", "iso2022-jp-3", "iso8859-1", "tis620", "cp874", "cp1252", "cp1251"]
    streaming_content_bypass: Literal["enable", "disable"]
    switching_protocols: Literal["bypass", "block"]
    unknown_http_version: Literal["reject", "tunnel", "best-effort"]
    http_0_9: Literal["allow", "block"]
    tunnel_non_http: Literal["enable", "disable"]
    h2c: Literal["enable", "disable"]
    unknown_content_encoding: Literal["block", "inspect", "bypass"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    stream_based_uncompressed_limit: int
    scan_bzip2: Literal["enable", "disable"]
    verify_dns_for_policy_matching: Literal["enable", "disable"]
    block_page_status_code: int
    retry_count: int
    domain_fronting: Literal["allow", "monitor", "block", "strict"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    ssl_offloaded: Literal["no", "yes"]
    address_ip_rating: Literal["enable", "disable"]


class ProfileProtocolOptionsFtpObject(FortiObject):
    """Nested object for ftp field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    options: Literal["clientcomfort", "oversize", "splice", "bypass-rest-command", "bypass-mode-command"]
    comfort_interval: int
    comfort_amount: int
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    stream_based_uncompressed_limit: int
    scan_bzip2: Literal["enable", "disable"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    ssl_offloaded: Literal["no", "yes"]
    explicit_ftp_tls: Literal["enable", "disable"]


class ProfileProtocolOptionsImapObject(FortiObject):
    """Nested object for imap field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsMapiObject(FortiObject):
    """Nested object for mapi field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]


class ProfileProtocolOptionsPop3Object(FortiObject):
    """Nested object for pop3 field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsSmtpObject(FortiObject):
    """Nested object for smtp field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["fragmail", "oversize", "splice"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    server_busy: Literal["enable", "disable"]
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsNntpObject(FortiObject):
    """Nested object for nntp field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    inspect_all: Literal["enable", "disable"]
    proxy_after_tcp_handshake: Literal["enable", "disable"]
    options: Literal["oversize", "splice"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]


class ProfileProtocolOptionsSshObject(FortiObject):
    """Nested object for ssh field with attribute access."""
    options: Literal["oversize", "clientcomfort", "servercomfort"]
    comfort_interval: int
    comfort_amount: int
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    stream_based_uncompressed_limit: int
    scan_bzip2: Literal["enable", "disable"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    ssl_offloaded: Literal["no", "yes"]


class ProfileProtocolOptionsDnsObject(FortiObject):
    """Nested object for dns field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]


class ProfileProtocolOptionsCifsObject(FortiObject):
    """Nested object for cifs field with attribute access."""
    ports: int | list[int]
    status: Literal["enable", "disable"]
    options: Literal["oversize"]
    oversize_limit: int
    uncompressed_oversize_limit: int
    uncompressed_nest_limit: int
    scan_bzip2: Literal["enable", "disable"]
    tcp_window_type: Literal["auto-tuning", "system", "static", "dynamic"]
    tcp_window_minimum: int
    tcp_window_maximum: int
    tcp_window_size: int
    server_credential_type: Literal["none", "credential-replication", "credential-keytab"]
    domain_controller: str
    server_keytab: str | list[str]


class ProfileProtocolOptionsMailsignatureObject(FortiObject):
    """Nested object for mail-signature field with attribute access."""
    status: Literal["disable", "enable"]
    signature: str


class ProfileProtocolOptionsObject(FortiObject):
    """Typed FortiObject for ProfileProtocolOptions with field access."""
    name: str
    comment: str
    replacemsg_group: str
    oversize_log: Literal["disable", "enable"]
    switching_protocols_log: Literal["disable", "enable"]
    http: ProfileProtocolOptionsHttpObject
    ftp: ProfileProtocolOptionsFtpObject
    imap: ProfileProtocolOptionsImapObject
    mapi: ProfileProtocolOptionsMapiObject
    pop3: ProfileProtocolOptionsPop3Object
    smtp: ProfileProtocolOptionsSmtpObject
    nntp: ProfileProtocolOptionsNntpObject
    ssh: ProfileProtocolOptionsSshObject
    dns: ProfileProtocolOptionsDnsObject
    cifs: ProfileProtocolOptionsCifsObject
    mail_signature: ProfileProtocolOptionsMailsignatureObject
    rpc_over_http: Literal["enable", "disable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class ProfileProtocolOptions:
    """
    
    Endpoint: firewall/profile_protocol_options
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileProtocolOptionsObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ProfileProtocolOptionsObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ProfileProtocolOptionsPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        replacemsg_group: str | None = ...,
        oversize_log: Literal["disable", "enable"] | None = ...,
        switching_protocols_log: Literal["disable", "enable"] | None = ...,
        http: ProfileProtocolOptionsHttpDict | None = ...,
        ftp: ProfileProtocolOptionsFtpDict | None = ...,
        imap: ProfileProtocolOptionsImapDict | None = ...,
        mapi: ProfileProtocolOptionsMapiDict | None = ...,
        pop3: ProfileProtocolOptionsPop3Dict | None = ...,
        smtp: ProfileProtocolOptionsSmtpDict | None = ...,
        nntp: ProfileProtocolOptionsNntpDict | None = ...,
        ssh: ProfileProtocolOptionsSshDict | None = ...,
        dns: ProfileProtocolOptionsDnsDict | None = ...,
        cifs: ProfileProtocolOptionsCifsDict | None = ...,
        mail_signature: ProfileProtocolOptionsMailsignatureDict | None = ...,
        rpc_over_http: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileProtocolOptionsObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ProfileProtocolOptionsPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        replacemsg_group: str | None = ...,
        oversize_log: Literal["disable", "enable"] | None = ...,
        switching_protocols_log: Literal["disable", "enable"] | None = ...,
        http: ProfileProtocolOptionsHttpDict | None = ...,
        ftp: ProfileProtocolOptionsFtpDict | None = ...,
        imap: ProfileProtocolOptionsImapDict | None = ...,
        mapi: ProfileProtocolOptionsMapiDict | None = ...,
        pop3: ProfileProtocolOptionsPop3Dict | None = ...,
        smtp: ProfileProtocolOptionsSmtpDict | None = ...,
        nntp: ProfileProtocolOptionsNntpDict | None = ...,
        ssh: ProfileProtocolOptionsSshDict | None = ...,
        dns: ProfileProtocolOptionsDnsDict | None = ...,
        cifs: ProfileProtocolOptionsCifsDict | None = ...,
        mail_signature: ProfileProtocolOptionsMailsignatureDict | None = ...,
        rpc_over_http: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileProtocolOptionsObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ProfileProtocolOptionsPayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        replacemsg_group: str | None = ...,
        oversize_log: Literal["disable", "enable"] | None = ...,
        switching_protocols_log: Literal["disable", "enable"] | None = ...,
        http: ProfileProtocolOptionsHttpDict | None = ...,
        ftp: ProfileProtocolOptionsFtpDict | None = ...,
        imap: ProfileProtocolOptionsImapDict | None = ...,
        mapi: ProfileProtocolOptionsMapiDict | None = ...,
        pop3: ProfileProtocolOptionsPop3Dict | None = ...,
        smtp: ProfileProtocolOptionsSmtpDict | None = ...,
        nntp: ProfileProtocolOptionsNntpDict | None = ...,
        ssh: ProfileProtocolOptionsSshDict | None = ...,
        dns: ProfileProtocolOptionsDnsDict | None = ...,
        cifs: ProfileProtocolOptionsCifsDict | None = ...,
        mail_signature: ProfileProtocolOptionsMailsignatureDict | None = ...,
        rpc_over_http: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "ProfileProtocolOptions",
    "ProfileProtocolOptionsPayload",
    "ProfileProtocolOptionsResponse",
    "ProfileProtocolOptionsObject",
]


"""
Pydantic Models for CMDB - firewall/address

Runtime validation models for firewall/address configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum
from uuid import UUID

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class AddressTaggingTags(BaseModel):
    """
    Child table model for tagging.tags.
    
    Tags.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="Tag name.")  # datasource: ['system.object-tagging.tags.name']
class AddressTagging(BaseModel):
    """
    Child table model for tagging.
    
    Config object tagging.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=63, default=None, description="Tagging entry name.")    
    category: str | None = Field(max_length=63, default=None, description="Tag category.")  # datasource: ['system.object-tagging.category']    
    tags: list[AddressTaggingTags] = Field(default_factory=list, description="Tags.")
class AddressSsoAttributeValue(BaseModel):
    """
    Child table model for sso-attribute-value.
    
    RADIUS attributes value.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=511, default=None, description="RADIUS attribute value.")
class AddressMacaddr(BaseModel):
    """
    Child table model for macaddr.
    
    Multiple MAC address ranges.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    macaddr: str = Field(max_length=127, description="MAC address ranges <start>[-<end>] separated by space.")
class AddressList(BaseModel):
    """
    Child table model for list.
    
    IP address list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    ip: str = Field(max_length=35, description="IP.")
class AddressFssoGroup(BaseModel):
    """
    Child table model for fsso-group.
    
    FSSO group(s).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=511, default=None, description="FSSO group name.")  # datasource: ['user.adgrp.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class AddressTypeEnum(str, Enum):
    """Allowed values for type_ field."""
    IPMASK = "ipmask"
    IPRANGE = "iprange"
    FQDN = "fqdn"
    GEOGRAPHY = "geography"
    WILDCARD = "wildcard"
    DYNAMIC = "dynamic"
    INTERFACE_SUBNET = "interface-subnet"
    MAC = "mac"
    ROUTE_TAG = "route-tag"

class AddressSubTypeEnum(str, Enum):
    """Allowed values for sub_type field."""
    SDN = "sdn"
    CLEARPASS_SPT = "clearpass-spt"
    FSSO = "fsso"
    RSSO = "rsso"
    EMS_TAG = "ems-tag"
    FORTIVOICE_TAG = "fortivoice-tag"
    FORTINAC_TAG = "fortinac-tag"
    SWC_TAG = "swc-tag"
    DEVICE_IDENTIFICATION = "device-identification"
    EXTERNAL_RESOURCE = "external-resource"
    OBSOLETE = "obsolete"

class AddressClearpassSptEnum(str, Enum):
    """Allowed values for clearpass_spt field."""
    UNKNOWN = "unknown"
    HEALTHY = "healthy"
    QUARANTINE = "quarantine"
    CHECKUP = "checkup"
    TRANSIENT = "transient"
    INFECTED = "infected"


# ============================================================================
# Main Model
# ============================================================================

class AddressModel(BaseModel):
    """
    Pydantic model for firewall/address configuration.
    
    Configure IPv4 addresses.
    
    Validation Rules:        - name: max_length=79 pattern=        - uuid: pattern=        - subnet: pattern=        - type_: pattern=        - route_tag: min=1 max=4294967295 pattern=        - sub_type: pattern=        - clearpass_spt: pattern=        - macaddr: pattern=        - start_ip: pattern=        - end_ip: pattern=        - fqdn: max_length=255 pattern=        - country: max_length=2 pattern=        - wildcard_fqdn: max_length=255 pattern=        - cache_ttl: min=0 max=86400 pattern=        - wildcard: pattern=        - sdn: max_length=35 pattern=        - fsso_group: pattern=        - sso_attribute_value: pattern=        - interface: max_length=35 pattern=        - tenant: max_length=35 pattern=        - organization: max_length=35 pattern=        - epg_name: max_length=255 pattern=        - subnet_name: max_length=255 pattern=        - sdn_tag: max_length=15 pattern=        - policy_group: max_length=15 pattern=        - obj_tag: max_length=255 pattern=        - obj_type: pattern=        - tag_detection_level: max_length=15 pattern=        - tag_type: max_length=63 pattern=        - hw_vendor: max_length=35 pattern=        - hw_model: max_length=35 pattern=        - os: max_length=35 pattern=        - sw_version: max_length=35 pattern=        - comment: max_length=255 pattern=        - associated_interface: max_length=35 pattern=        - color: min=0 max=32 pattern=        - filter_: max_length=2047 pattern=        - sdn_addr_type: pattern=        - node_ip_only: pattern=        - obj_id: max_length=255 pattern=        - list_: pattern=        - tagging: pattern=        - allow_routing: pattern=        - passive_fqdn_learning: pattern=        - fabric_object: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=79, default=None, description="Address name.")    
    uuid: str | None = Field(default="00000000-0000-0000-0000-000000000000", description="Universally Unique Identifier (UUID; automatically assigned but can be manually reset).")    
    subnet: Any = Field(default="0.0.0.0 0.0.0.0", description="IP address and subnet mask of address.")    
    type_: AddressTypeEnum | None = Field(default=AddressTypeEnum.IPMASK, serialization_alias="type", description="Type of address.")    
    route_tag: int | None = Field(ge=1, le=4294967295, default=0, description="route-tag address.")    
    sub_type: AddressSubTypeEnum | None = Field(default=AddressSubTypeEnum.SDN, description="Sub-type of address.")    
    clearpass_spt: AddressClearpassSptEnum | None = Field(default=AddressClearpassSptEnum.UNKNOWN, description="SPT (System Posture Token) value.")    
    macaddr: list[AddressMacaddr] = Field(default_factory=list, description="Multiple MAC address ranges.")    
    start_ip: str | None = Field(default="0.0.0.0", description="First IP address (inclusive) in the range for the address.")    
    end_ip: str | None = Field(default="0.0.0.0", description="Final IP address (inclusive) in the range for the address.")    
    fqdn: str | None = Field(max_length=255, default=None, description="Fully Qualified Domain Name address.")    
    country: str | None = Field(max_length=2, default=None, description="IP addresses associated to a specific country.")    
    wildcard_fqdn: str | None = Field(max_length=255, default=None, description="Fully Qualified Domain Name with wildcard characters.")    
    cache_ttl: int | None = Field(ge=0, le=86400, default=0, description="Defines the minimal TTL of individual IP addresses in FQDN cache measured in seconds.")    
    wildcard: Any = Field(default="0.0.0.0 0.0.0.0", description="IP address and wildcard netmask.")    
    sdn: str | None = Field(max_length=35, default=None, description="SDN.")  # datasource: ['system.sdn-connector.name']    
    fsso_group: list[AddressFssoGroup] = Field(default_factory=list, description="FSSO group(s).")    
    sso_attribute_value: list[AddressSsoAttributeValue] = Field(default_factory=list, description="RADIUS attributes value.")    
    interface: str = Field(max_length=35, description="Name of interface whose IP address is to be used.")  # datasource: ['system.interface.name']    
    tenant: str | None = Field(max_length=35, default=None, description="Tenant.")    
    organization: str | None = Field(max_length=35, default=None, description="Organization domain name (Syntax: organization/domain).")    
    epg_name: str | None = Field(max_length=255, default=None, description="Endpoint group name.")    
    subnet_name: str | None = Field(max_length=255, default=None, description="Subnet name.")    
    sdn_tag: str | None = Field(max_length=15, default=None, description="SDN Tag.")    
    policy_group: str | None = Field(max_length=15, default=None, description="Policy group name.")    
    obj_tag: str | None = Field(max_length=255, default=None, description="Tag of dynamic address object.")    
    obj_type: Literal["ip", "mac"] | None = Field(default="ip", description="Object type.")    
    tag_detection_level: str | None = Field(max_length=15, default=None, description="Tag detection level of dynamic address object.")    
    tag_type: str | None = Field(max_length=63, default=None, description="Tag type of dynamic address object.")    
    hw_vendor: str | None = Field(max_length=35, default=None, description="Dynamic address matching hardware vendor.")    
    hw_model: str | None = Field(max_length=35, default=None, description="Dynamic address matching hardware model.")    
    os: str | None = Field(max_length=35, default=None, description="Dynamic address matching operating system.")    
    sw_version: str | None = Field(max_length=35, default=None, description="Dynamic address matching software version.")    
    comment: str | None = Field(max_length=255, default=None, description="Comment.")    
    associated_interface: str | None = Field(max_length=35, default=None, description="Network interface associated with address.")  # datasource: ['system.interface.name', 'system.zone.name']    
    color: int | None = Field(ge=0, le=32, default=0, description="Color of icon on the GUI.")    
    filter_: str = Field(max_length=2047, serialization_alias="filter", description="Match criteria filter.")    
    sdn_addr_type: Literal["private", "public", "all"] | None = Field(default="private", description="Type of addresses to collect.")    
    node_ip_only: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable collection of node addresses only in Kubernetes.")    
    obj_id: str | None = Field(max_length=255, default=None, description="Object ID for NSX.")    
    list_: list[AddressList] = Field(default_factory=list, serialization_alias="list", description="IP address list.")    
    tagging: list[AddressTagging] = Field(default_factory=list, description="Config object tagging.")    
    allow_routing: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable use of this address in routing configurations.")    
    passive_fqdn_learning: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable passive learning of FQDNs.  When enabled, the FortiGate learns, trusts, and saves FQDNs from endpoint DNS queries (default = enable).")    
    fabric_object: Literal["enable", "disable"] | None = Field(default="disable", description="Security Fabric global object setting.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('sdn')
    @classmethod
    def validate_sdn(cls, v: Any) -> Any:
        """
        Validate sdn field.
        
        Datasource: ['system.sdn-connector.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('associated_interface')
    @classmethod
    def validate_associated_interface(cls, v: Any) -> Any:
        """
        Validate associated_interface field.
        
        Datasource: ['system.interface.name', 'system.zone.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "AddressModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_sdn_references(self, client: Any) -> list[str]:
        """
        Validate sdn references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/sdn-connector        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AddressModel(
            ...     sdn="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_sdn_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "sdn", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.sdn_connector.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Sdn '{value}' not found in "
                "system/sdn-connector"
            )        
        return errors    
    async def validate_fsso_group_references(self, client: Any) -> list[str]:
        """
        Validate fsso_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/adgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AddressModel(
            ...     fsso_group=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_fsso_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "fsso_group", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.user.adgrp.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Fsso-Group '{value}' not found in "
                    "user/adgrp"
                )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AddressModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_associated_interface_references(self, client: Any) -> list[str]:
        """
        Validate associated_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        - system/zone        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AddressModel(
            ...     associated_interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_associated_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "associated_interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        elif await client.api.cmdb.system.zone.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Associated-Interface '{value}' not found in "
                "system/interface or system/zone"
            )        
        return errors    
    async def validate_tagging_references(self, client: Any) -> list[str]:
        """
        Validate tagging references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/object-tagging        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = AddressModel(
            ...     tagging=[{"category": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_tagging_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "tagging", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("category")
            else:
                value = getattr(item, "category", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.object_tagging.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Tagging '{value}' not found in "
                    "system/object-tagging"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_sdn_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_fsso_group_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_associated_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_tagging_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "AddressModel",    "AddressMacaddr",    "AddressFssoGroup",    "AddressSsoAttributeValue",    "AddressList",    "AddressTagging",    "AddressTagging.Tags",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:01.474153Z
# ============================================================================


"""FortiOS CMDB - Fortiguard category"""

from .clear_statistics import ClearStatistics
from .manual_update import ManualUpdate
from .server_info import ServerInfo
from .test_availability import TestAvailability
from .update import Update

__all__ = [
    "ClearStatistics",
    "Fortiguard",
    "ManualUpdate",
    "ServerInfo",
    "TestAvailability",
    "Update",
]


class Fortiguard:
    """Fortiguard endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Fortiguard endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.clear_statistics = ClearStatistics(client)
        self.manual_update = ManualUpdate(client)
        self.server_info = ServerInfo(client)
        self.test_availability = TestAvailability(client)
        self.update = Update(client)



"""FortiOS CMDB - AutoConfig category"""

from .custom import Custom
from .default import Default
from .policy import Policy

__all__ = [
    "AutoConfig",
    "Custom",
    "Default",
    "Policy",
]


class AutoConfig:
    """AutoConfig endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """AutoConfig endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.custom = Custom(client)
        self.default = Default(client)
        self.policy = Policy(client)
